@@ -1,12 +1,14 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 use crate::common::util::{AtPath, TestScenario, UCommand};
 use once_cell::sync::Lazy;
 use std::fs::{metadata, set_permissions, OpenOptions, Permissions};
 use std::os::unix::fs::{OpenOptionsExt, PermissionsExt};
 use std::sync::Mutex;
 
-extern crate chmod;
-extern crate libc;
-use self::libc::umask;
+use libc::umask;
 
 static TEST_FILE: &str = "file";
 static REFERENCE_FILE: &str = "reference";
@@ -276,7 +278,7 @@ fn test_chmod_many_options() {
 #[test]
 #[allow(clippy::unreadable_literal)]
 fn test_chmod_reference_file() {
-    let tests = vec![
+    let tests = [
         TestCase {
             args: vec!["--reference", REFERENCE_FILE, TEST_FILE],
             before: 0o100070,
@@ -540,6 +542,7 @@ fn test_invalid_arg() {
 }
 
 #[test]
+#[cfg(not(target_os = "android"))]
 fn test_mode_after_dash_dash() {
     let (at, ucmd) = at_and_ucmd!();
     run_single_test(
@@ -642,6 +645,20 @@ fn test_quiet_n_verbose_used_multiple_times() {
         .succeeds();
 }
 
+#[test]
+fn test_changes_from_identical_reference() {
+    let scene = TestScenario::new(util_name!());
+    let at = &scene.fixtures;
+    at.touch("file");
+    scene
+        .ucmd()
+        .arg("-c")
+        .arg("--reference=file")
+        .arg("file")
+        .succeeds()
+        .no_stdout();
+}
+
 #[test]
 fn test_gnu_invalid_mode() {
     let scene = TestScenario::new(util_name!());
@@ -651,6 +668,7 @@ fn test_gnu_invalid_mode() {
 }
 
 #[test]
+#[cfg(not(target_os = "android"))]
 fn test_gnu_options() {
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
